#import "SearchController.h"
#import "MyDocument.h"

@implementation SearchController

- (id)init
{
	_query = [[NSMetadataQuery alloc] init];
	
	// Restrict the search to our working directory
	NSString *path = [@"~/Library/QNotes" stringByExpandingTildeInPath];
	[_query setSearchScopes:[NSArray arrayWithObjects:path,nil]];

	// Tell the query to sort according to relevancy and age
	[_query setSortDescriptors:[NSArray arrayWithObjects:
			[[[NSSortDescriptor alloc] initWithKey:NSMetadataQueryResultContentRelevanceAttribute ascending:NO] autorelease],
			[[[NSSortDescriptor alloc] initWithKey:(id)kMDItemContentModificationDate ascending:NO] autorelease],
			nil]];
	
	// Register for notifications when the query has initially 
	// gathered its data
	_notifCenter = [NSNotificationCenter defaultCenter];
	[_notifCenter addObserver:self 
					 selector:@selector(queryFinishedGathering:) 
			 name:@"NSMetadataQueryDidFinishGatheringNotification" 
		 object:nil];
	
	return self;
}

// When the gathering phase is over, we can stop the query.
- (void)queryFinishedGathering:(NSNotification*)notification {
	[_query stopQuery];
}

- (void)dealloc {
	[_query stopQuery];
	[_notifCenter removeObserver:self];
    [_query release];
    [_searchKey release];
    [super dealloc];
}

- (IBAction)orderOutSearchPanel:(id)sender
{
	if (searchPanel == nil) {
		if (![NSBundle loadNibNamed:@"SearchPanel" owner:self])  {
            NSLog(@"Failed to load SearchPanel.nib");
            NSBeep();
            return;
        }
	}
	if (![_query isStopped]) {
		[_query stopQuery];
	}
	[searchPanel makeKeyAndOrderFront:sender];
}


// In the nib file, we will bind to "query.results" in the ArrayControler
- (NSMetadataQuery *)query {
    return _query;
}

// In the nib file, the NSSearchField's value is bound to "searchKey"
- (NSString *)searchKey {
    return [[_searchKey copy] autorelease];
}

// The search key gets set from the search box through bindings
- (void)setSearchKey:(NSString *) value {
    if (_searchKey != value) {
        [_searchKey release];
        if (value == nil) value = @"";
        _searchKey = [value copy];
        [self runTheQuery];
    }
}

- (void) runTheQuery
{
	// set predicate
	// in future this will depend on the search menu selection
	NSString *predicateFormat = @"kMDItemTextContent like[cd] %@";	
	NSString *textType = @"kMDItemContentType == 'public.plain-text'";

	if (_searchKey != nil ) {
		NSPredicate *p = [NSPredicate predicateWithFormat:predicateFormat,
								   [_searchKey stringByAppendingString:@"*"]];
		NSPredicate * subPredicate = [NSPredicate predicateWithFormat:
								  textType];
		p = [NSCompoundPredicate andPredicateWithSubpredicates:
                     [NSArray arrayWithObjects:p, subPredicate, nil]];
    
		[_query setPredicate:p];
	
		// and go ..
	
		[_query startQuery];
	}
	
}

// Let the user stop the query if neeed.
- (void) killQuery:(id)sender
{
	[_query stopQuery];
	
}

// Open a selected item
- (void)openFile:(NSString *)path
{
	[_query stopQuery];
	[[NSWorkspace sharedWorkspace] openFile:path withApplication:@"QNote.app"];
	[searchPanel close];
}
@end
